%-------------------------------------------------------%
% 				  Image segmentation examination            % 
% 							March 2, 2001                      %
%          Department of Computer Engineering           %
%              Khon Kaen University                     %
%-------------------------------------------------------%

% Problem statement
% To extract the boundary of the object, we need to 2 things
% 	1. The location of the boundary
% 	2. The direction of the boundary.
%
% In a vector form, we need 2 vector fields to extract the boundary
% 	1. the tangential velocity field
%  2. the normal compressive velocity field.
%
% The first field tells the direction of the boundary while
% the second field tells the location of the boundary.
%
% The important property of the normal compressive velocity field
% is that all vectors in the field must point to the nearest boundary.
%
% In this example, we obtained the tangential velocity field
% by rotating the gradient field by 90 degree while the normal
% compress velocity field is obtained by multiplying the gradient
% field with the Laplacian image (see example of mathlab codes below).
%
% *********************** Homework ******************************
% There are many ways to generate the normal compressive velocity 
% field. In this problem, replace the original mathlab codes in 
% Lines 100-111 with your own mathlab codes for computing
% the normal compressive velocity field (do not copy the original codes).
%
% Hint:
% Some important property 
% 	1. Don't forget! The normal velocity field must has all vectors
% 	   point to the nearest boundary.
% 	2. Gradient always points to the steepest direction of the image.
%  3. The gradient image sqrt(px.^2+py.^2) creates the "mountain"
%     enclosing the object.
%  4. You can create the normal compressive velocity field
%     from the gradient image (using your own idea "how to do it")
%  5. It's very simple. You need only 2-3 mathlab commands to do it.
%
% When you finish writing the code, play around with the program and
% see the difference between the result obtained from the original
% code and the result from your code. 
% 
% Note that, you must put files segmentexam.m, drawsignbw.m,
% drawvfield.m, streamflow.m, arrow.m and rice.tif in the 
% same directory before running the program.
%****************************************************************

% Mathlab code begin

%read image files
p = imread('rice.tif');
p = double(p);

% crop image
p  = p(81:115,51:85);

% plot image
figure(1)
clf
imagesc(p)
title('Original image : rice seed')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
pause

%Compute derivatives of an image
% px = the first derivative is x-direction
% py = the first derivative is y-direction
% pxx = the second derivative is x-direction
% pyy = the second derivative is y-direction
[px, py] = gradient(p); 
[pxx,pxy] = gradient(px);
[pxy,pyy] = gradient(py);

%Laplacian image
pl = pxx+pyy;

% Compute tangential velocity field
% The tangent field is obtained by rotating 
% the gradient field by 90 degree 
% tx = x-component of the tangent field
% ty = y component of the tangent field
tx = -py;
ty =  px;  
t = sqrt(tx.^2+ty.^2);

% Normalize the tangent field so that the maximum magnitude = 1
tx = tx/max(max(t));
ty = ty/max(max(t));

% Please change the mathlab code between these two dash lines
% for computing your own normal compressive velocity field
%---------------------------------------------------------------------%
% Begin 

% Compute a normal compressive velocity field 
nx = px.*pl; % old code
ny = py.*pl; % old code






% End
%---------------------------------------------------------------------%



% Normalize the normal field so that the maximum magnitude = 1
n = sqrt(nx.^2+ny.^2);
nx = nx/max(max(n));
ny = ny/max(max(n));

figure(1)
clf
imagesc(px)
title('Derivative in X-direction')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
pause

figure(1)
clf
imagesc(py)
title('Derivative in Y-direction')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
pause

figure(1)
clf
imagesc(sqrt(px.^2+py.^2))
title('Gradient image')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
pause

figure(1)
imagesc(p)
title('Gradient field')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
drawvfield(px,py)
pause

figure(1)
clf
imagesc(pl)
title('Laplacian image')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
pause

figure(2)
imagesc(pl)
title('Sign of Laplacian image')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
drawsignbw(pl,0.5)
pause


figure(3)
imagesc(p)
title('Tangential velocity field')
xlabel('Place mouse cursor at the boundary and click left button to see the result');
set(gca,'DataAspectRatio',[1 1 1])
set(gcf,'WindowButtonDownFcn','streamflow(tx,ty);')  
colormap(gray)
drawvfield(tx,ty)
pause

figure(4)
imagesc(p)
title('Normal compressive velocity field')
set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
drawvfield(nx,ny)
pause

figure(5)
imagesc(p)
title('tangent + normal fields')
xlabel('Place mouse cursor at the boundary and click left button to see the result');

set(gca,'DataAspectRatio',[1 1 1])
colormap(gray)
drawvfield(tx+nx,ty+ny)
set(gcf,'WindowButtonDownFcn','streamflow(tx+nx,ty+ny);')